﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Optimization.Web.Models;
using HeuristicLab.Services.Optimization.Billing.Model;
using HeuristicLab.Services.Optimization.Billing.Interfaces;
using HeuristicLab.Services.Optimization.Billing;
using System.Web.Security;

namespace HeuristicLab.Services.Optimization.Web.Controllers
{
    [Authorize(Roles = "Web User")]
    public class OrderController : Controller
    {
        private IOptimizationBilling billing = new MockupOptimizationBilling();

        //
        // GET: /Order/

        public ActionResult Index()
        {
          return CreateOrder(new OrderModel()); //View();
        }


        [HttpPost]
        public ActionResult CreateOrder(OrderModel model) {
          Session["order"] = model;
          return AddProduct();
        }

        public ActionResult AddProduct() {
          return View("AddProduct", new ProductsModel() { Products = billing.GetProducts() });
        }

        [HttpPost]
        public ActionResult AddProduct(ProductsModel product) {
          var order = Session["order"] as OrderModel;
          var prod = (from b in billing.GetProducts() 
                      where b.Id == product.ProductId
                      select b).FirstOrDefault();
          order.ProductQuantities[prod.Id] = product.Quantity;
          return AddProduct();
        }

        public ActionResult SaveOrder() {
          var orderModel = Session["order"] as OrderModel;
          foreach (var key in orderModel.ProductQuantities.Keys) {
            var product = (from p in billing.GetProducts() where p.Id == key select p).FirstOrDefault();
            orderModel.Products[key] = product;            
          }
          return View(orderModel);
        }

        [HttpPost]
        [ActionName("SaveOrder")]
        public ActionResult SaveOrderPost() {
          var orderModel = Session["order"] as OrderModel;
          //TODO: Save order via back-end
          var order = new Order() {
            State = OrderState.Created,
            User = new User() { Name = Membership.GetUser().UserName },
            BillingType = orderModel.BillingType,
            BillingPeriod = orderModel.BillingPeriod
          };
          
          var orderLines = new List<OrderLine>();
          var products = billing.GetProducts();
          foreach (var key in orderModel.ProductQuantities.Keys) {
            var product = (from p in products where p.Id == key select p).FirstOrDefault();
            orderLines.Add(new OrderLine() {
               Order = order,
               Product = product,
               ProductPrice = product.Price,
               Quantity = orderModel.ProductQuantities[key]
            });
          }
          order.OrderLines = orderLines;
          billing.SaveOrder(order);
          return RedirectToAction("OrderSaved");
        }

        public ActionResult OrderSaved() {
          return View();
        }
    }
}
